﻿using Microsoft.AspNetCore.Mvc;
using WCS.BLL.Services.IService;
using WCS.BLL.Services.Service;
using WCS.Model;
using WCS.Model.ApiModel;
using WCS.Model.ApiModel.MXL4;
using WCS.Model.ApiModel.SelfCheck;
using WCS.Model.ApiModel.SingleLight;
using WCS.Model.ApiModel.Stocktaking;
using WCS.Model.ApiModel.User;
using WCS.Model.WebSocketModel;

namespace WCS.WebApi.Controllers
{
    /// <summary>
    /// 液晶面板 煤科院钻探分院项目
    /// </summary>
    [ApiController]
    [Route("[controller]")]
    public class MXL4Controller : ControllerBase
    {

        public IMXL4Service _mxl4Service { get; set; }

        public MXL4Controller(IMXL4Service mxl4Service)
        {
            _mxl4Service = mxl4Service;
        }

        [Route("sysOrderMXL4")]
        [HttpPost(Name = "sysOrderMXL4")]
        public async Task<ResponseBase> sysOrderMXL4(SysOrderMXL4Request request)
        {
            try
            {
                return await _mxl4Service.sysOrderMXL4(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "操作失败：" + ex.Message,
                };
            }
        }

        /// <summary>
        /// 刷新库存信息 当上游系统入库后可以调用此接口对硬件显示的库存信息进行刷新
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("refreshInventory")]
        [HttpPost(Name = "refreshInventory")]
        public async Task<ResponseCommon> refreshInventory(RefreshInventoryRequest request)
        {
            try
            {
                //校验传入的参数
                if (request.StoreCodes == null || request.StoreCodes.Count == 0)
                { 
                    return new ResponseCommon()
                    {
                        Code = 201,
                        Message = "操作失败：缺少需要刷新的库位！",
                    };
                }
                return await _mxl4Service.refreshInventoryRequest(request);
            }
            catch (Exception ex)
            {
                return new ResponseCommon()
                {
                    Code = 300,
                    Message = "操作失败：" + ex.Message,
                };
            }
        }


        #region 调试用的API
        [Route("reSetAll")]
        [HttpPost(Name = "reSetAll")]
        public async Task<ResponseBase> reSetAll()
        {
            try
            {
                return await _mxl4Service.reSetAll();
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "操作失败：" + ex.Message,
                };
            }
        }

        [Route("defaultDisplay")]
        [HttpPost(Name = "defaultDisplay")]
        public async Task<ResponseBase> defaultDisplay()
        {
            try
            {
                return await _mxl4Service.defaultDisplay();
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "操作失败：" + ex.Message,
                };
            }
        }
        #endregion
    }
}
